#include <string>
#include <map>
#include <vector>
#include <algorithm>
#include <iterator>
#include <iostream>
#include <functional>
#include <sstream>
#include <utility>

class Matcher //: public X509_Object::Search_Func
   {
   public:
      bool operator()(const std::string& arg, const std::string& val) const
         {
         return (arg.find(looking_for) != std::string::npos);
         }

      Matcher(const std::string& looking_for)
         { this->looking_for = looking_for; }
   protected:
      std::string looking_for;
   };

class Information_Store
   {
   public:
      std::multimap<std::string, std::string> get_info(const Matcher&) const;
      std::vector<std::string> get_info(const std::string&) const;

      void add(const std::string, const std::string);
      void add(const std::multimap<std::string, std::string>&);

      std::string print() const;
   private:
      std::multimap<std::string, std::string> info;
   };

std::multimap<std::string, std::string>
Information_Store::get_info(const Matcher& matcher) const
   {
   std::multimap<std::string, std::string> out;

   std::multimap<std::string, std::string>::const_iterator i;
   i = info.begin();
   while(i != info.end())
      {
      if(matcher(i->first, i->second))
         out.insert(*i);
      ++i;
      }

   return out;
   }

std::vector<std::string>
Information_Store::get_info(const std::string& looking_for) const
   {
   std::vector<std::string> out;

   std::multimap<std::string, std::string>::const_iterator i;
   i = info.begin();
   while(i != info.end())
      {
      if(i->first == looking_for)
         out.push_back(i->second);
      ++i;
      }

   return out;
   }

void Information_Store::add(const std::string arg, const std::string val)
   {
   info.insert(std::make_pair(arg, val));
   }

void Information_Store::add(const std::multimap<std::string, std::string>& in)
   {
   std::multimap<std::string, std::string>::const_iterator i = in.begin();
   while(i != in.end())
      {
      info.insert(*i);
      ++i;
      }
   }

std::string Information_Store::print() const
   {
   std::multimap<std::string, std::string>::const_iterator i;
   i = info.begin();
   std::string out;

   while(i != info.end())
      {
      out += i->first + " = " + i->second + "\n";
      ++i;
      }

   return out;
   }

namespace std {

ostream& operator<<(ostream& out, const std::pair<std::string, std::string>& in)
   {
   out << in.first << " = " << in.second;
   return out;
   }

}

std::string join(const std::multimap<std::string, std::string>& in)
   {
   std::ostringstream out;
   std::copy(in.begin(), in.end(),
             std::ostream_iterator<std::pair<std::string, std::string> >(out, "\n"));
   return out.str();
   }

std::string join(const std::vector<std::string>& in)
   {
   std::ostringstream out;
   std::copy(in.begin(), in.end(),
             std::ostream_iterator<std::string>(out, "\n"));
   return out.str();
   }

std::string join1(const std::multimap<std::string, std::string>& in)
   {
   using namespace std;

   map<string, vector<string> > rep;
      {
      multimap<string, string>::const_iterator i = in.begin();
      while(i != in.end())
         {
         rep[i->first].push_back(i->second);
         ++i;
         }
      }

   ostringstream out;
      {
      map<string, vector<string> >::const_iterator i = rep.begin();
      while(i != rep.end())
         {
         out << i->first << " = ";
         vector<string>::const_iterator j = i->second.begin();
         while(j != i->second.end())
            {
            out << *j;
            ++j;
            if(j != i->second.end())
               out << ", ";
            }
         out << "\n";
         ++i;
         }
      }

   return out.str();
   }

int main()
   {
   Information_Store store;

   store.add("x509.basic.is_ca", "false");
   store.add("x509.subject.dn.name", "jack");
   store.add("x509.subject.dn.name", "lloyd");
   store.add("x509.subject.dn.org", "PRION");
   store.add("x509.subject.dn.org", "GronCorp");
   store.add("x509.subject.dn.orgunit", "Mathematical Munitions");
   store.add("x509.policy", "1.2.4.5");
   store.add("x509.policy", "1.2.4.7");

   //std::cout << store.print();

   std::cout << join(store.get_info("x509.policy")) << std::endl;
   std::cout << join1(store.get_info(Matcher("x509.policy"))) << std::endl;
   std::cout << join1(store.get_info(Matcher("x509.subject.dn."))) << std::endl;
   }
